package com.tencent.devops.auth.constant

/**
 * 流水线微服务模块请求返回状态码
 * 返回码制定规则（除开0代表成功外，为了兼容历史接口成功状态都是返回0）：
 * 1、返回码总长度为7位，
 * 2、前2位数字代表系统名称（如21代表持续集成平台）
 * 3、第3位和第4位数字代表微服务模块（00：common-公共模块 01：process-流水线 02：artifactory-版本仓库 03:dispatch-分发 04：dockerhost-docker机器
 *    05:environment-持续集成环境 06：experience-版本体验 07：image-镜像 08：log-持续集成日志 09：measure-度量 10：monitoring-监控 11：notify-通知
 *    12：openapi-开放api接口 13：plugin-插件 14：quality-质量红线 15：repository-代码库 16：scm-软件配置管理 17：support-持续集成支撑服务
 *    18：ticket-证书凭据 19：project-项目管理 20：store-商店 21： auth-权限）
 * 4、最后3位数字代表具体微服务模块下返回给客户端的业务逻辑含义（如001代表系统服务繁忙，建议一个模块一类的返回码按照一定的规则制定）
 * 5、系统公共的返回码写在CommonMessageCode这个类里面，具体微服务模块的返回码写在相应模块的常量类里面
 *
 * @since: 2018-12-21
 * @version: $Revision$ $Date$ $LastChangedBy$
 *
 */
object AuthMessageCode {
    const val GROUP_EXIST = "2121001" // 权限系统： 用户组已存在
    const val GROUP_NOT_EXIST = "2121002" // 权限系统： 用户组不存在
    const val GROUP_NOT_BIND_PERSSION = "2121003" // 权限系统：自定义用户组未绑定权限
    const val GROUP_USER_ALREADY_EXIST = "2121004" // 权限系统：用户已在该用户组
    const val GROUP_ACTION_EMPTY = "2121005" // 权限系统：用户组未绑定动作
    const val TOKEN_TICKET_FAIL = "2121106" // 权限系统：token校验失败
    const val PARENT_TYPE_FAIL = "2121107" // 权限系统：父类资源必须为"项目"
}