/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.tencent.devops.common.pipeline.enums

enum class PipelineSvgType {
    NEVER, // 从来没有构建
    SUCCEEDED, // 构建成功
    FAILED; // 构建失败

    companion object {
        fun getSvgContent(type: PipelineSvgType?): String {
            return when (type) {
                NEVER -> NEVER_SVG
                SUCCEEDED -> SUCCEEDED_SVG
                FAILED -> FAILED_SVG
                null -> NEVER_SVG
            }
        }

        private const val NEVER_SVG = """<?xml version="1.0" encoding="utf-8"?>
<!-- Generator: Adobe Illustrator 22.1.0, SVG Export Plug-In . SVG Version: 6.00 Build 0)  -->
<svg version="1.1" id="图层_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
	 viewBox="0 0 171.7 20" style="enable-background:new 0 0 171.7 20;" xml:space="preserve" height="20">
<style type="text/css">
	.st0{clip-path:url(#SVGID_2_);fill:#555555;}
	.st1{clip-path:url(#SVGID_4_);fill:#FF9600;}
	.st2{clip-path:url(#SVGID_6_);fill:url(#SVGID_7_);}
	.st3{fill:#FFFFFF;}
	.st4{display:none;}
	.st5{display:inline;}
	.st6{display:inline;fill:#FFFFFF;}
</style>
<g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_1_" d="M3,0h165.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_2_">
							<use xlink:href="#SVGID_1_"  style="overflow:visible;"/>
						</clipPath>
						<rect class="st0" width="171.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_3_" d="M3,0h165.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_4_">
							<use xlink:href="#SVGID_3_"  style="overflow:visible;"/>
						</clipPath>
						<rect x="98" class="st1" width="73.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_5_" d="M3,0h165.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_6_">
							<use xlink:href="#SVGID_5_"  style="overflow:visible;"/>
						</clipPath>

							<linearGradient id="SVGID_7_" gradientUnits="userSpaceOnUse" x1="-2194.4502" y1="100.2" x2="-2194.4502" y2="101.2" gradientTransform="matrix(182.7 0 0 20 401011.9062 -2004)">
							<stop  offset="0" style="stop-color:#000000;stop-opacity:0"/>
							<stop  offset="1" style="stop-color:#000000;stop-opacity:0.2"/>
						</linearGradient>
						<rect class="st2" width="171.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
</g>
<g>
	<g>
		<path d="M23.9,14.4V5.8h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,7.6,30,8c0,0.4-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.8
			c0.5,0.2,0.9,0.4,1.2,0.8s0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4s-0.8,0.1-1.4,0.1L23.9,14.4
			L23.9,14.4z M25.1,9.4H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,8.5,29,8.1c0-0.3-0.1-0.5-0.2-0.8S28.3,7,28.1,7
			s-0.7-0.1-1.3-0.1h-1.7V9.4z M25.1,13.4h2.1c0.4,0,0.6,0,0.8,0c0.3,0,0.5-0.1,0.7-0.2s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7
			c0-0.3-0.1-0.6-0.2-0.8c-0.2-0.2-0.4-0.4-0.7-0.5s-0.7-0.1-1.3-0.1h-2V13.4z"/>
		<path d="M31.9,14.4V5.8H33v4.3l4.3-4.3h1.5l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3L33,11.5v3L31.9,14.4L31.9,14.4z"/>
		<path d="M43.3,14.4V5.8h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8s0.3,0.7,0.3,1.2c0,0.7-0.2,1.3-0.7,1.9
			S47.9,11,46.7,11h-2.2v3.5h-1.2V14.4z M44.4,9.9h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2c0-0.4-0.1-0.7-0.3-0.9S48,7,47.7,6.9
			c-0.2-0.1-0.5-0.1-1.1-0.1h-2.2V9.9z"/>
		<path d="M51.2,7V5.8h1.1V7H51.2z M51.2,14.4V8.2h1.1v6.2H51.2z"/>
		<path d="M53.8,16.8V8.2h1V9c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2s0.3,1,0.3,1.6
			s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3H53.8z M54.8,11.3
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8S58,9.9,57.7,9.5S57,8.9,56.5,8.9c-0.4,0-0.8,0.2-1.2,0.6
			S54.8,10.5,54.8,11.3z"/>
		<path d="M64.8,12.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S62.1,8,63,8c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.6,12.9,64.8,12.4z M61.3,10.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S61.4,10.2,61.3,10.7z"/>
		<path d="M67.2,14.4V5.8h1.1v8.6C68.3,14.4,67.2,14.4,67.2,14.4z"/>
		<path d="M69.9,7V5.8H71V7H69.9z M69.9,14.4V8.2H71v6.2H69.9z"/>
		<path d="M72.5,14.4V8.2h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8c0,0.2,0.1,0.5,0.1,1v3.8
			h-1.1v-3.8c0-0.4,0-0.8-0.1-1S76.2,9.3,76,9.2S75.5,9,75.2,9c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4H72.5z"/>
		<path d="M83.5,12.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S80.8,8,81.7,8c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S83.3,12.9,83.5,12.4z M80,10.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S80,10.2,80,10.7z"/>
		<path d="M85.4,12.6l1-0.2c0.1,0.4,0.2,0.7,0.5,1c0.3,0.2,0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7c0-0.2-0.1-0.4-0.3-0.6
			c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5s-0.5-0.3-0.7-0.6s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8s0.3-0.4,0.5-0.6
			c0.2-0.1,0.4-0.2,0.7-0.3C87.3,8,87.6,8,87.9,8c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1
			c0-0.3-0.2-0.6-0.4-0.8s-0.5-0.3-1-0.3S87.2,8.9,87,9c-0.2,0.2-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3
			c0.1,0,0.4,0.1,0.9,0.3c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7
			s-0.8,0.3-1.3,0.3c-0.8,0-1.4-0.2-1.8-0.5S85.6,13.2,85.4,12.6z"/>
	</g>
	<g>
		<path class="st3" d="M23.9,13.4V4.8h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,6.6,30,7s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.8
			C29.3,9,29.7,9.3,30,9.6s0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4s-0.8,0.1-1.4,0.1L23.9,13.4
			L23.9,13.4z M25.1,8.4H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,7.5,29,7.1c0-0.3-0.1-0.5-0.2-0.8S28.3,6,28.1,6
			s-0.7-0.1-1.3-0.1h-1.7V8.4z M25.1,12.4h2.1c0.4,0,0.6,0,0.8,0c0.3,0,0.5-0.1,0.7-0.2s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7
			c0-0.3-0.1-0.6-0.2-0.8s-0.4-0.4-0.7-0.5S27.6,9.4,27,9.4h-2v3H25.1z"/>
		<path class="st3" d="M31.9,13.4V4.8H33v4.3l4.3-4.3h1.5l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3L33,10.5v3L31.9,13.4L31.9,13.4z"/>
		<path class="st3" d="M43.3,13.4V4.8h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8s0.3,0.7,0.3,1.2
			c0,0.7-0.2,1.3-0.7,1.9S47.9,10,46.7,10h-2.2v3.5h-1.2V13.4z M44.4,8.9h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2
			c0-0.4-0.1-0.7-0.3-0.9S48,6,47.7,5.9c-0.2-0.1-0.5-0.1-1.1-0.1h-2.2V8.9z"/>
		<path class="st3" d="M51.2,6V4.8h1.1V6H51.2z M51.2,13.4V7.2h1.1v6.2H51.2z"/>
		<path class="st3" d="M53.8,15.8V7.2h1V8c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2
			s0.3,1,0.3,1.6s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3H53.8z M54.8,10.3
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8S58,8.9,57.7,8.5S57,7.9,56.5,7.9c-0.4,0-0.8,0.2-1.2,0.6
			S54.8,9.5,54.8,10.3z"/>
		<path class="st3" d="M64.8,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S62.1,7,63,7c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.6,11.9,64.8,11.4z M61.3,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S61.4,9.2,61.3,9.7z"/>
		<path class="st3" d="M67.2,13.4V4.8h1.1v8.6C68.3,13.4,67.2,13.4,67.2,13.4z"/>
		<path class="st3" d="M69.9,6V4.8H71V6H69.9z M69.9,13.4V7.2H71v6.2H69.9z"/>
		<path class="st3" d="M72.5,13.4V7.2h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8
			c0,0.2,0.1,0.5,0.1,1v3.8h-1.1V9.6c0-0.4,0-0.8-0.1-1S76.2,8.3,76,8.2S75.5,8,75.2,8c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4
			H72.5z"/>
		<path class="st3" d="M83.5,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S80.8,7,81.7,7c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S83.3,11.9,83.5,11.4z M80,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S80,9.2,80,9.7z"/>
		<path class="st3" d="M85.5,11.6l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7
			c0-0.2-0.1-0.4-0.3-0.6c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5S86,9.9,85.9,9.7s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8
			s0.3-0.4,0.5-0.6c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1
			c0-0.3-0.2-0.6-0.4-0.8s-0.5-0.3-1-0.3s-0.8,0.1-1,0.2c-0.2,0.1-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3
			c0.1,0,0.4,0.1,0.9,0.3c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7
			s-0.8,0.3-1.3,0.3c-0.8,0-1.4-0.2-1.8-0.5S85.6,12.2,85.5,11.6z"/>
	</g>
</g>
<g>
	<g>
		<path d="M106,14.5V8.3h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8c0,0.2,0.1,0.5,0.1,1v3.8
			h-1v-3.8c0-0.4,0-0.8-0.1-1s-0.2-0.4-0.4-0.5S109,9,108.8,9c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4H106V14.5z"/>
		<path d="M116.9,12.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S116.8,12.9,116.9,12.5z M113.5,10.8h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S113.5,10.2,113.5,10.8z"/>
		<path d="M121.1,14.5l-2.4-6.2h1.1l1.3,3.7c0.1,0.4,0.3,0.8,0.4,1.3c0.1-0.3,0.2-0.7,0.4-1.2l1.4-3.8h1.1l-2.4,6.2H121.1z"/>
		<path d="M129.6,12.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3H126c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S129.4,12.9,129.6,12.5z M126.1,10.8h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S126.2,10.2,126.1,10.8z"/>
		<path d="M132,14.5V8.3h0.9v0.9c0.2-0.4,0.5-0.7,0.7-0.9s0.4-0.2,0.7-0.2c0.4,0,0.7,0.1,1.1,0.3l-0.4,1c-0.3-0.2-0.5-0.2-0.8-0.2
			c-0.2,0-0.4,0.1-0.6,0.2s-0.3,0.3-0.4,0.6c-0.1,0.4-0.2,0.8-0.2,1.2v3.3C133,14.5,132,14.5,132,14.5z"/>
		<path d="M140.3,14.5h-1V5.9h1.1V9c0.4-0.6,1-0.8,1.7-0.8c0.4,0,0.7,0.1,1.1,0.2s0.6,0.4,0.8,0.7s0.4,0.6,0.5,1s0.2,0.8,0.2,1.3
			c0,1.1-0.3,1.9-0.8,2.5s-1.2,0.9-1.9,0.9s-1.3-0.3-1.7-0.9L140.3,14.5L140.3,14.5z M140.3,11.3c0,0.7,0.1,1.3,0.3,1.6
			c0.3,0.5,0.8,0.8,1.3,0.8s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8s-0.2-1.4-0.5-1.8S142.5,9,142,9s-0.9,0.2-1.2,0.6S140.3,10.6,140.3,11.3z
			"/>
		<path d="M150.1,14.5v-0.9c-0.5,0.7-1.1,1.1-2,1.1c-0.4,0-0.7-0.1-1-0.2s-0.6-0.3-0.7-0.5s-0.3-0.5-0.3-0.8c0-0.2-0.1-0.5-0.1-1
			V8.3h1.1v3.5c0,0.6,0,0.9,0.1,1.1c0.1,0.3,0.2,0.5,0.4,0.7s0.5,0.2,0.8,0.2s0.6-0.1,0.9-0.2s0.5-0.4,0.6-0.7s0.2-0.7,0.2-1.2V8.3
			h1.1v6.2H150.1z"/>
		<path d="M152.7,7.1V5.9h1.1v1.2C153.8,7.1,152.7,7.1,152.7,7.1z M152.7,14.5V8.3h1.1v6.2H152.7z"/>
		<path d="M155.3,14.5V5.9h1.1v8.6H155.3z"/>
		<path d="M162.1,14.5v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7c0-0.6,0.1-1.2,0.3-1.7
			s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6v-3h1v8.6C163,14.5,162.1,14.5,162.1,14.5z M158.7,11.4
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9S160.9,9,160.4,9s-0.9,0.2-1.2,0.6
			S158.7,10.6,158.7,11.4z"/>
	</g>
	<g>
		<path class="st3" d="M106,13.4V7.2h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8
			c0,0.2,0.1,0.5,0.1,1v3.8h-1V9.6c0-0.4,0-0.8-0.1-1s-0.2-0.4-0.4-0.5S109,8,108.7,8c-0.4,0-0.8,0.1-1.2,0.4S107,9.2,107,10v3.4
			H106z"/>
		<path class="st3" d="M116.9,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S116.8,11.9,116.9,11.4z M113.5,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S113.5,9.2,113.5,9.7z"/>
		<path class="st3" d="M121.1,13.4l-2.4-6.2h1.1l1.3,3.7c0.1,0.4,0.3,0.8,0.4,1.3c0.1-0.3,0.2-0.7,0.4-1.2l1.4-3.8h1.1l-2.4,6.2
			H121.1z"/>
		<path class="st3" d="M129.6,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3H126c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S129.4,11.9,129.6,11.4z M126.1,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S126.2,9.2,126.1,9.7z"/>
		<path class="st3" d="M132,13.4V7.2h0.9v0.9c0.2-0.4,0.5-0.7,0.7-0.9S134,7,134.3,7c0.4,0,0.7,0.1,1.1,0.3l-0.4,1
			c-0.3-0.2-0.5-0.2-0.8-0.2c-0.2,0-0.4,0.1-0.6,0.2s-0.3,0.3-0.4,0.6c-0.1,0.4-0.2,0.8-0.2,1.2v3.3C133,13.4,132,13.4,132,13.4z"/>
		<path class="st3" d="M140.3,13.4h-1V4.8h1.1v3.1c0.4-0.6,1-0.8,1.7-0.8c0.4,0,0.7,0.1,1.1,0.2s0.6,0.4,0.8,0.7s0.4,0.6,0.5,1
			s0.2,0.8,0.2,1.3c0,1.1-0.3,1.9-0.8,2.5s-1.2,0.9-1.9,0.9s-1.3-0.3-1.7-0.9L140.3,13.4L140.3,13.4z M140.3,10.3
			c0,0.7,0.1,1.3,0.3,1.6c0.3,0.5,0.8,0.8,1.3,0.8s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8s-0.2-1.4-0.5-1.8s-0.7-0.6-1.2-0.6
			s-0.9,0.2-1.2,0.6S140.3,9.5,140.3,10.3z"/>
		<path class="st3" d="M150.1,13.4v-0.9c-0.5,0.7-1.1,1.1-2,1.1c-0.4,0-0.7-0.1-1-0.2s-0.6-0.3-0.7-0.5s-0.3-0.5-0.3-0.8
			c0-0.2-0.1-0.5-0.1-1V7.2h1.1v3.5c0,0.6,0,0.9,0.1,1.1c0.1,0.3,0.2,0.5,0.4,0.7s0.5,0.2,0.8,0.2s0.6-0.1,0.9-0.2s0.5-0.4,0.6-0.7
			s0.2-0.7,0.2-1.2V7.2h1.1v6.2H150.1z"/>
		<path class="st3" d="M152.7,6V4.8h1.1V6H152.7z M152.7,13.4V7.2h1.1v6.2H152.7z"/>
		<path class="st3" d="M155.3,13.4V4.8h1.1v8.6C156.4,13.4,155.3,13.4,155.3,13.4z"/>
		<path class="st3" d="M162.1,13.4v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7
			c0-0.6,0.1-1.2,0.3-1.7s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6v-3h1v8.6H162.1z M158.7,10.3
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9s-0.7-0.6-1.2-0.6s-0.9,0.2-1.2,0.6
			S158.7,9.5,158.7,10.3z"/>
	</g>
</g>
<g class="st4">
	<path class="st5" d="M18,7.9c0-1.7-1.3-3-3-3h-0.5V4.1c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,3.9,7.5,4,7.5,4.1v2.7c0,0,0.1,0.1,0.2,0.1
		h6.7c0.1,0,0.2-0.1,0.2-0.2V5.9H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V9.1c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2v0.8H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5c0,0.1,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V14c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,10.8,18,9.5,18,7.9z"
		/>
	<path class="st6" d="M18,6.9c0-1.7-1.3-3-3-3h-0.5V3.1c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,2.9,7.5,3,7.5,3.1v2.7c0,0,0.1,0.1,0.2,0.1
		h6.7c0.1,0,0.2-0.1,0.2-0.2V4.9H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V8.1c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2v0.8H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5c0,0.1,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V13c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,9.8,18,8.5,18,6.9z"/>
</g>
<g>
	<path d="M18,8c0-1.7-1.3-3-3-3h-0.5V4.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,4,7.5,4.1,7.5,4.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V6H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V9.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V10H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,17,7.6,17,7.7,17h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,10.9,18,9.6,18,8z"/>
	<path class="st3" d="M18,7c0-1.7-1.3-3-3-3h-0.5V3.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,3,7.5,3.1,7.5,3.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V5H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V8.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V9H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,16,7.6,16,7.7,16h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,9.9,18,8.6,18,7z"/>
</g>
</svg>
"""

        private const val SUCCEEDED_SVG = """<?xml version="1.0" encoding="utf-8"?>
<!-- Generator: Adobe Illustrator 22.1.0, SVG Export Plug-In . SVG Version: 6.00 Build 0)  -->
<svg version="1.1" id="图层_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
	 viewBox="0 0 169.8 20" style="enable-background:new 0 0 169.8 20;" xml:space="preserve" height="20">
<style type="text/css">
	.st0{clip-path:url(#SVGID_2_);fill:#555555;}
	.st1{clip-path:url(#SVGID_4_);fill:#00C873;}
	.st2{clip-path:url(#SVGID_6_);fill:url(#SVGID_7_);}
	.st3{fill:#FFFFFF;}
</style>
<g>
	<g>
		<g>
			<g>
				<g>
					<defs>
						<path id="SVGID_1_" d="M3,0h163.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
					</defs>
					<clipPath id="SVGID_2_">
						<use xlink:href="#SVGID_1_"  style="overflow:visible;"/>
					</clipPath>
					<rect class="st0" width="169.7" height="20"/>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<defs>
						<path id="SVGID_3_" d="M3,0h163.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
					</defs>
					<clipPath id="SVGID_4_">
						<use xlink:href="#SVGID_3_"  style="overflow:visible;"/>
					</clipPath>
					<rect x="98" class="st1" width="71.7" height="20"/>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<defs>
						<path id="SVGID_5_" d="M3,0h163.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
					</defs>
					<clipPath id="SVGID_6_">
						<use xlink:href="#SVGID_5_"  style="overflow:visible;"/>
					</clipPath>

						<linearGradient id="SVGID_7_" gradientUnits="userSpaceOnUse" x1="-2748.4072" y1="397.75" x2="-2748.4072" y2="396.75" gradientTransform="matrix(182.7 0 0 -20 502218.8438 7955)">
						<stop  offset="0" style="stop-color:#000000;stop-opacity:0"/>
						<stop  offset="1" style="stop-color:#000000;stop-opacity:0.2"/>
					</linearGradient>
					<rect class="st2" width="169.7" height="20"/>
				</g>
			</g>
		</g>
	</g>
</g>
<g>
	<g>
		<path d="M24,14.5V6h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,7.8,30,8.2s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.8
			c0.5,0.2,0.9,0.4,1.2,0.8c0.3,0.4,0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4s-0.8,0.1-1.4,0.1
			C27.2,14.5,24,14.5,24,14.5z M25.2,9.6H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,8.6,29,8.3s-0.1-0.5-0.2-0.8
			s-0.3-0.4-0.6-0.4S27.5,7,26.9,7h-1.7C25.2,7,25.2,9.6,25.2,9.6z M25.2,13.5h2.1c0.4,0,0.6,0,0.8,0c0.3,0,0.5-0.1,0.7-0.2
			s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7c0-0.3-0.1-0.6-0.2-0.8c-0.2-0.2-0.4-0.4-0.7-0.5s-0.7-0.1-1.3-0.1h-2V13.5z"/>
		<path d="M32,14.5V6h1.1v4.3L37.5,6H39l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3l-1.4,1.4v3H32V14.5z"/>
		<path d="M43.4,14.5V6h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8S50,8,50,8.4c0,0.7-0.2,1.3-0.7,1.9
			S48,11,46.8,11h-2.2v3.5H43.4z M44.6,10h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2c0-0.4-0.1-0.7-0.3-0.9S48.1,7.1,47.8,7
			c-0.2,0-0.5,0-1,0h-2.2V10z"/>
		<path d="M51.3,7.2V6h1.1v1.2H51.3z M51.3,14.5V8.3h1.1v6.2H51.3z"/>
		<path d="M54,16.9V8.3h1v0.8c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2s0.3,1,0.3,1.6
			s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3H54z M54.9,11.5
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8s-0.2-1.4-0.5-1.8S57.1,9,56.6,9c-0.4,0-0.8,0.2-1.2,0.6
			S54.9,10.7,54.9,11.5z"/>
		<path d="M64.9,12.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.7,13,64.9,12.5z M61.4,10.8h3.5c0-0.5-0.2-0.9-0.4-1.2C64.2,9.2,63.7,9,63.2,9
			S62.3,9.2,62,9.5S61.5,10.3,61.4,10.8z"/>
		<path d="M67.3,14.5V6h1.1v8.6h-1.1V14.5z"/>
		<path d="M70,7.2V6h1v1.2H70z M70,14.5V8.3h1v6.2H70z"/>
		<path d="M72.6,14.5V8.3h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8c0,0.2,0.1,0.5,0.1,1v3.8
			h-1.1v-3.8c0-0.4,0-0.8-0.1-1S76.2,9.3,76,9.2S75.5,9,75.3,9c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4h-1
			C72.6,14.4,72.6,14.5,72.6,14.5z"/>
		<path d="M83.6,12.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8S79,12.5,79,11.5s0.3-1.9,0.8-2.4
			s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6s0.8,0.5,1.3,0.5
			c0.4,0,0.7-0.1,1-0.3S83.4,13,83.6,12.5z M80.1,10.8h3.5c0-0.5-0.2-0.9-0.4-1.2C82.9,9.2,82.4,9,81.9,9S81,9.2,80.7,9.5
			S80.2,10.3,80.1,10.8z"/>
		<path d="M85.6,12.7l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7c0-0.2-0.1-0.4-0.3-0.6
			c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5S86.2,11,86,10.7s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8c0.1-0.2,0.3-0.4,0.5-0.6
			c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2S90,8.7,90.1,9s0.3,0.6,0.4,1l-1,0.1c0-0.3-0.2-0.6-0.4-0.8
			S88.5,9,88.1,9c-0.5,0-0.8,0.1-1,0.2s-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3c0.1,0,0.4,0.1,0.9,0.3
			c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7s-0.8,0.3-1.3,0.3
			c-0.8,0-1.4-0.2-1.8-0.5C86,13.8,85.7,13.3,85.6,12.7z"/>
	</g>
	<g>
		<path class="st3" d="M24,13.5V5h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,6.8,30,7.2s-0.1,0.7-0.3,1
			c-0.2,0.3-0.4,0.6-0.8,0.8c0.5,0.2,0.9,0.4,1.2,0.8s0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4
			s-0.8,0.1-1.4,0.1C27.3,13.5,24,13.5,24,13.5z M25.2,8.6H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,7.6,29,7.3
			c0-0.3-0.1-0.5-0.2-0.8s-0.3-0.4-0.6-0.4S27.5,6,26.9,6h-1.7C25.2,6,25.2,8.6,25.2,8.6z M25.2,12.5h2.1c0.4,0,0.6,0,0.8,0
			c0.3,0,0.5-0.1,0.7-0.2s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7c0-0.3-0.1-0.6-0.2-0.8s-0.4-0.4-0.7-0.5s-0.7-0.1-1.3-0.1h-2V12.5z"/>
		<path class="st3" d="M32,13.5V5h1.1v4.3L37.5,5H39l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3l-1.4,1.4v3H32V13.5z"/>
		<path class="st3" d="M43.4,13.5V5h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8S50,7,50,7.4
			c0,0.7-0.2,1.3-0.7,1.9S48,10,46.8,10h-2.2v3.5H43.4z M44.6,9h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2c0-0.4-0.1-0.7-0.3-0.9
			S48.1,6.1,47.8,6c-0.2,0-0.5,0-1,0h-2.2V9z"/>
		<path class="st3" d="M51.3,6.2V5h1.1v1.2H51.3z M51.3,13.5V7.3h1.1v6.2H51.3z"/>
		<path class="st3" d="M54,15.9V7.3h1v0.8c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2
			s0.3,1,0.3,1.6s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3
			C55.1,15.9,54,15.9,54,15.9z M54.9,10.5c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8s-0.2-1.4-0.5-1.8
			S57.1,8,56.6,8c-0.4,0-0.8,0.2-1.2,0.6S54.9,9.7,54.9,10.5z"/>
		<path class="st3" d="M64.9,11.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8S66,9.4,66,10.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.7,12,64.9,11.5z M61.4,9.8h3.5c0-0.5-0.2-0.9-0.4-1.2C64.2,8.2,63.7,8,63.2,8
			S62.3,8.2,62,8.5S61.5,9.3,61.4,9.8z"/>
		<path class="st3" d="M67.3,13.5V5h1.1v8.6h-1.1V13.5z"/>
		<path class="st3" d="M70,6.2V5h1v1.2H70z M70,13.5V7.3h1v6.2H70z"/>
		<path class="st3" d="M72.6,13.5V7.3h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8
			c0,0.2,0.1,0.5,0.1,1v3.8h-1.1V9.8c0-0.4,0-0.8-0.1-1S76.2,8.4,76,8.3s-0.5-0.2-0.7-0.2c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6
			v3.4H72.6z"/>
		<path class="st3" d="M83.6,11.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8S79,11.5,79,10.5
			s0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S83.4,12,83.6,11.5z M80.1,9.8h3.5c0-0.5-0.2-0.9-0.4-1.2C82.9,8.2,82.4,8,81.9,8
			S81,8.2,80.7,8.5S80.2,9.3,80.1,9.8z"/>
		<path class="st3" d="M85.6,11.7l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7
			c0-0.2-0.1-0.4-0.3-0.6c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5S86.2,10,86,9.7S85.8,9.3,85.8,9s0.1-0.5,0.2-0.8
			s0.3-0.4,0.5-0.6c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2S90,7.7,90.1,8s0.3,0.6,0.4,1l-1,0.1
			c0-0.3-0.2-0.6-0.4-0.8S88.5,8,88.1,8c-0.5,0-0.8,0.1-1,0.2s-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3
			c0.1,0,0.4,0.1,0.9,0.3c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6c0.2,0.2,0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7
			s-0.8,0.3-1.3,0.3c-0.8,0-1.4-0.2-1.8-0.5S85.7,12.3,85.6,11.7z"/>
	</g>
</g>
<g>
	<g>
		<path d="M105.7,12.8l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7c0-0.2-0.1-0.4-0.3-0.6
			c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5s-0.5-0.3-0.7-0.6s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8s0.3-0.4,0.5-0.6
			c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1c0-0.3-0.2-0.6-0.4-0.8
			s-0.5-0.3-1-0.3s-0.8,0.1-1,0.2S107,9.5,107,9.8c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3c0.1,0,0.4,0.1,0.9,0.3
			c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7s-0.8,0.3-1.3,0.3
			c-0.8,0-1.4-0.2-1.8-0.5S105.8,13.4,105.7,12.8z"/>
		<path d="M116.2,14.6v-0.9c-0.5,0.7-1.1,1.1-2,1.1c-0.4,0-0.7-0.1-1-0.2s-0.6-0.3-0.7-0.5s-0.3-0.5-0.3-0.8c0-0.2-0.1-0.5-0.1-1
			V8.4h1.1v3.5c0,0.6,0,0.9,0.1,1.1c0.1,0.3,0.2,0.5,0.4,0.7s0.5,0.2,0.8,0.2s0.6-0.1,0.9-0.2s0.5-0.4,0.6-0.7s0.2-0.7,0.2-1.2V8.4
			h1.1v6.2C117.3,14.6,116.2,14.6,116.2,14.6z"/>
		<path d="M122.9,12.3l1,0.1c-0.1,0.7-0.4,1.3-0.9,1.7s-1,0.6-1.7,0.6c-0.9,0-1.5-0.3-2.1-0.8s-0.8-1.4-0.8-2.4
			c0-0.7,0.1-1.3,0.3-1.8s0.6-0.9,1-1.1s1-0.4,1.5-0.4c0.7,0,1.2,0.2,1.7,0.5s0.7,0.8,0.8,1.5l-1,0.2c-0.1-0.4-0.3-0.7-0.5-0.9
			s-0.5-0.3-0.9-0.3c-0.5,0-1,0.2-1.3,0.6s-0.5,1-0.5,1.8s0.2,1.4,0.5,1.8s0.7,0.6,1.3,0.6c0.4,0,0.8-0.1,1-0.4
			S122.8,12.9,122.9,12.3z"/>
		<path d="M128.9,12.3l1,0.1c-0.1,0.7-0.4,1.3-0.9,1.7s-1,0.6-1.7,0.6c-0.9,0-1.5-0.3-2.1-0.8s-0.8-1.4-0.8-2.4
			c0-0.7,0.1-1.3,0.3-1.8s0.6-0.9,1-1.1s1-0.4,1.5-0.4c0.7,0,1.2,0.2,1.7,0.5s0.7,0.8,0.8,1.5l-1,0.2c-0.1-0.4-0.3-0.7-0.5-0.9
			s-0.5-0.3-0.9-0.3c-0.5,0-1,0.2-1.3,0.6s-0.5,1-0.5,1.8s0.2,1.4,0.5,1.8s0.7,0.6,1.3,0.6c0.4,0,0.8-0.1,1-0.4
			S128.8,12.9,128.9,12.3z"/>
		<path d="M135.1,12.6l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S134.9,13.1,135.1,12.6z M131.6,10.9h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S131.6,10.4,131.6,10.9z"/>
		<path d="M141.7,12.6l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S141.6,13.1,141.7,12.6z M138.3,10.9h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S138.3,10.4,138.3,10.9z"/>
		<path d="M148.2,14.6v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7c0-0.6,0.1-1.2,0.3-1.7
			s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6V6h1v8.6H148.2z M144.8,11.5c0,0.8,0.2,1.4,0.5,1.8
			s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9s-0.7-0.6-1.2-0.6s-0.9,0.2-1.2,0.6S144.8,10.7,144.8,11.5z
			"/>
		<path d="M155.1,12.6l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S154.9,13.1,155.1,12.6z M151.6,10.9h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S151.7,10.4,151.6,10.9z"/>
		<path d="M161.5,14.6v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7c0-0.6,0.1-1.2,0.3-1.7
			s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6V6h1v8.6H161.5z M158.2,11.5c0,0.8,0.2,1.4,0.5,1.8
			s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9s-0.7-0.6-1.2-0.6s-0.9,0.2-1.2,0.6S158.2,10.7,158.2,11.5z
			"/>
	</g>
	<g>
		<path class="st3" d="M105.7,11.7l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7
			c0-0.2-0.1-0.4-0.3-0.6c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5s-0.5-0.3-0.7-0.6s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8
			s0.3-0.4,0.5-0.6c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1
			c0-0.3-0.2-0.6-0.4-0.8s-0.5-0.3-1-0.3s-0.8,0.1-1,0.2c-0.2,0.1-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3
			c0.1,0,0.4,0.1,0.9,0.3c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7
			s-0.8,0.3-1.3,0.3c-0.8,0-1.4-0.2-1.8-0.5S105.8,12.3,105.7,11.7z"/>
		<path class="st3" d="M116.2,13.5v-0.9c-0.5,0.7-1.1,1.1-2,1.1c-0.4,0-0.7-0.1-1-0.2s-0.6-0.3-0.7-0.5s-0.3-0.5-0.3-0.8
			c0-0.2-0.1-0.5-0.1-1V7.3h1.1v3.5c0,0.6,0,0.9,0.1,1.1c0.1,0.3,0.2,0.5,0.4,0.7s0.5,0.2,0.8,0.2s0.6-0.1,0.9-0.2s0.5-0.4,0.6-0.7
			s0.2-0.7,0.2-1.2V7.3h1.1v6.2H116.2z"/>
		<path class="st3" d="M122.9,11.3l1,0.1c-0.1,0.7-0.4,1.3-0.9,1.7s-1,0.6-1.7,0.6c-0.9,0-1.5-0.3-2.1-0.8s-0.8-1.4-0.8-2.4
			c0-0.7,0.1-1.3,0.3-1.8s0.6-0.9,1-1.1s1-0.4,1.5-0.4c0.7,0,1.2,0.2,1.7,0.5s0.7,0.8,0.8,1.5l-1,0.2c-0.1-0.4-0.3-0.7-0.5-0.9
			S121.7,8,121.4,8c-0.5,0-1,0.2-1.3,0.6s-0.5,1-0.5,1.8s0.2,1.4,0.5,1.8s0.7,0.6,1.3,0.6c0.4,0,0.8-0.1,1-0.4
			S122.8,11.8,122.9,11.3z"/>
		<path class="st3" d="M128.9,11.3l1,0.1c-0.1,0.7-0.4,1.3-0.9,1.7s-1,0.6-1.7,0.6c-0.9,0-1.5-0.3-2.1-0.8s-0.8-1.4-0.8-2.4
			c0-0.7,0.1-1.3,0.3-1.8s0.6-0.9,1-1.1s1-0.4,1.5-0.4c0.7,0,1.2,0.2,1.7,0.5s0.7,0.8,0.8,1.5l-1,0.2c-0.1-0.4-0.3-0.7-0.5-0.9
			S127.7,8,127.4,8c-0.5,0-1,0.2-1.3,0.6s-0.5,1-0.5,1.8s0.2,1.4,0.5,1.8s0.7,0.6,1.3,0.6c0.4,0,0.8-0.1,1-0.4
			S128.8,11.8,128.9,11.3z"/>
		<path class="st3" d="M135.1,11.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S134.9,12,135.1,11.5z M131.6,9.8h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S131.6,9.3,131.6,9.8z"/>
		<path class="st3" d="M141.7,11.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S141.6,12,141.7,11.5z M138.3,9.8h3.5c0-0.5-0.2-0.9-0.4-1.2C141,8.2,140.6,8,140,8
			c-0.5,0-0.9,0.2-1.2,0.5S138.3,9.3,138.3,9.8z"/>
		<path class="st3" d="M148.2,13.5v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7
			c0-0.6,0.1-1.2,0.3-1.7s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6V5h1v8.6h-0.9V13.5z M144.8,10.4
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9S147,8,146.5,8s-0.9,0.2-1.2,0.6
			S144.8,9.6,144.8,10.4z"/>
		<path class="st3" d="M155.1,11.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S154.9,12,155.1,11.5z M151.6,9.8h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S151.7,9.3,151.6,9.8z"/>
		<path class="st3" d="M161.5,13.5v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7
			c0-0.6,0.1-1.2,0.3-1.7s0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6V5h1v8.6h-0.9V13.5z M158.2,10.4
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9S160.4,8,159.9,8s-0.9,0.2-1.2,0.6
			S158.2,9.6,158.2,10.4z"/>
	</g>
</g>
<g>
	<path d="M18,8c0-1.7-1.3-3-3-3h-0.5V4.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,4,7.5,4.1,7.5,4.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V6H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V9.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V10H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,17,7.6,17,7.7,17h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,10.9,18,9.6,18,8z"/>
	<path class="st3" d="M18,7c0-1.7-1.3-3-3-3h-0.5V3.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,3,7.5,3.1,7.5,3.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V5H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V8.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V9H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,16,7.6,16,7.7,16h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,9.9,18,8.6,18,7z"/>
</g>
</svg>
"""

        private const val FAILED_SVG = """<?xml version="1.0" encoding="utf-8"?>
<!-- Generator: Adobe Illustrator 22.1.0, SVG Export Plug-In . SVG Version: 6.00 Build 0)  -->
<svg version="1.1" id="图层_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
	 viewBox="0 0 142.8 20" style="enable-background:new 0 0 142.8 20;" xml:space="preserve"  height="20">
<style type="text/css">
	.st0{clip-path:url(#SVGID_2_);fill:#555555;}
	.st1{clip-path:url(#SVGID_4_);fill:#FF5656;}
	.st2{clip-path:url(#SVGID_6_);fill:url(#SVGID_7_);}
	.st3{fill:#FFFFFF;}
</style>
<g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_1_" d="M3,0h136.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_2_">
							<use xlink:href="#SVGID_1_"  style="overflow:visible;"/>
						</clipPath>
						<rect class="st0" width="142.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_3_" d="M3,0h136.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_4_">
							<use xlink:href="#SVGID_3_"  style="overflow:visible;"/>
						</clipPath>
						<rect x="98" class="st1" width="44.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
	<g>
		<g>
			<g>
				<g>
					<g>
						<defs>
							<path id="SVGID_5_" d="M3,0h136.7c1.7,0,3,1.3,3,3v14c0,1.7-1.3,3-3,3H3c-1.7,0-3-1.3-3-3V3C0,1.3,1.3,0,3,0z"/>
						</defs>
						<clipPath id="SVGID_6_">
							<use xlink:href="#SVGID_5_"  style="overflow:visible;"/>
						</clipPath>

							<linearGradient id="SVGID_7_" gradientUnits="userSpaceOnUse" x1="-2239.2834" y1="100.2" x2="-2239.2834" y2="101.2" gradientTransform="matrix(182.7 0 0 20 409188.4062 -2004)">
							<stop  offset="0" style="stop-color:#000000;stop-opacity:0"/>
							<stop  offset="1" style="stop-color:#000000;stop-opacity:0.2"/>
						</linearGradient>
						<rect class="st2" width="142.7" height="20"/>
					</g>
				</g>
			</g>
		</g>
	</g>
</g>
<g>
	<g>
		<path d="M23.9,14.4V5.8h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,7.6,30,8c0,0.4-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.8
			c0.5,0.2,0.9,0.4,1.2,0.8s0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4s-0.8,0.1-1.4,0.1L23.9,14.4
			L23.9,14.4z M25.1,9.4H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,8.5,29,8.1c0-0.3-0.1-0.5-0.2-0.8S28.3,7,28.1,7
			s-0.7-0.1-1.3-0.1h-1.7V9.4z M25.1,13.4h2.1c0.4,0,0.6,0,0.8,0c0.3,0,0.5-0.1,0.7-0.2s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7
			c0-0.3-0.1-0.6-0.2-0.8c-0.2-0.2-0.4-0.4-0.7-0.5s-0.7-0.1-1.3-0.1h-2V13.4z"/>
		<path d="M31.9,14.4V5.8H33v4.3l4.3-4.3h1.5l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3L33,11.5v3L31.9,14.4L31.9,14.4z"/>
		<path d="M43.3,14.4V5.8h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8s0.3,0.7,0.3,1.2c0,0.7-0.2,1.3-0.7,1.9
			S47.9,11,46.7,11h-2.2v3.5h-1.2V14.4z M44.4,9.9h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2c0-0.4-0.1-0.7-0.3-0.9S48,7,47.7,6.9
			c-0.2-0.1-0.5-0.1-1.1-0.1h-2.2V9.9z"/>
		<path d="M51.2,7V5.8h1.1V7H51.2z M51.2,14.4V8.2h1.1v6.2H51.2z"/>
		<path d="M53.8,16.8V8.2h1V9c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2s0.3,1,0.3,1.6
			s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3H53.8z M54.8,11.3
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8S58,9.9,57.7,9.5S57,8.9,56.5,8.9c-0.4,0-0.8,0.2-1.2,0.6
			S54.8,10.5,54.8,11.3z"/>
		<path d="M64.8,12.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S62.1,8,63,8c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.6,12.9,64.8,12.4z M61.3,10.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S61.4,10.2,61.3,10.7z"/>
		<path d="M67.2,14.4V5.8h1.1v8.6C68.3,14.4,67.2,14.4,67.2,14.4z"/>
		<path d="M69.9,7V5.8H71V7H69.9z M69.9,14.4V8.2H71v6.2H69.9z"/>
		<path d="M72.5,14.4V8.2h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8c0,0.2,0.1,0.5,0.1,1v3.8
			h-1.1v-3.8c0-0.4,0-0.8-0.1-1S76.2,9.3,76,9.2S75.5,9,75.2,9c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4H72.5z"/>
		<path d="M83.5,12.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S80.8,8,81.7,8c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S83.3,12.9,83.5,12.4z M80,10.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S80,10.2,80,10.7z"/>
		<path d="M85.5,12.6l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7c0-0.2-0.1-0.4-0.3-0.6
			c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5s-0.5-0.3-0.7-0.6s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8s0.3-0.4,0.5-0.6
			c0.2-0.1,0.4-0.2,0.7-0.3S87.7,8,88,8c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1c0-0.3-0.2-0.6-0.4-0.8
			s-0.5-0.3-1-0.3s-0.8,0.1-1,0.2s-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3c0.1,0,0.4,0.1,0.9,0.3
			c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7s-0.8,0.3-1.3,0.3
			c-0.8,0-1.4-0.2-1.8-0.5S85.6,13.2,85.5,12.6z"/>
	</g>
	<g>
		<path class="st3" d="M23.9,13.4V4.8h3.2c0.7,0,1.2,0.1,1.6,0.3s0.7,0.4,0.9,0.8S30,6.6,30,7s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.8
			C29.3,9,29.7,9.3,30,9.6s0.4,0.8,0.4,1.3c0,0.4-0.1,0.8-0.3,1.1s-0.4,0.6-0.6,0.8s-0.6,0.3-0.9,0.4s-0.8,0.1-1.4,0.1L23.9,13.4
			L23.9,13.4z M25.1,8.4H27c0.5,0,0.9,0,1.1-0.1c0.3-0.1,0.5-0.2,0.7-0.4S29,7.5,29,7.1c0-0.3-0.1-0.5-0.2-0.8S28.3,6,28.1,6
			s-0.7-0.1-1.3-0.1h-1.7V8.4z M25.1,12.4h2.1c0.4,0,0.6,0,0.8,0c0.3,0,0.5-0.1,0.7-0.2s0.3-0.3,0.4-0.5s0.2-0.5,0.2-0.7
			c0-0.3-0.1-0.6-0.2-0.8s-0.4-0.4-0.7-0.5S27.6,9.4,27,9.4h-2v3H25.1z"/>
		<path class="st3" d="M31.9,13.4V4.8H33v4.3l4.3-4.3h1.5l-3.6,3.5l3.8,5.1h-1.5l-3.1-4.3L33,10.5v3L31.9,13.4L31.9,13.4z"/>
		<path class="st3" d="M43.3,13.4V4.8h3.2c0.6,0,1,0,1.3,0.1c0.4,0.1,0.8,0.2,1.1,0.4s0.5,0.5,0.7,0.8s0.3,0.7,0.3,1.2
			c0,0.7-0.2,1.3-0.7,1.9S47.9,10,46.7,10h-2.2v3.5h-1.2V13.4z M44.4,8.9h2.2c0.7,0,1.3-0.1,1.6-0.4s0.5-0.7,0.5-1.2
			c0-0.4-0.1-0.7-0.3-0.9S48,6,47.7,5.9c-0.2-0.1-0.5-0.1-1.1-0.1h-2.2V8.9z"/>
		<path class="st3" d="M51.2,6V4.8h1.1V6H51.2z M51.2,13.4V7.2h1.1v6.2H51.2z"/>
		<path class="st3" d="M53.8,15.8V7.2h1V8c0.2-0.3,0.5-0.6,0.8-0.7s0.6-0.2,1-0.2c0.5,0,1,0.1,1.4,0.4s0.7,0.7,0.9,1.2
			s0.3,1,0.3,1.6s-0.1,1.2-0.3,1.7s-0.6,0.9-1,1.2s-0.9,0.4-1.4,0.4c-0.4,0-0.7-0.1-0.9-0.2s-0.5-0.3-0.7-0.6v3H53.8z M54.8,10.3
			c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.8S58,8.9,57.7,8.5S57,7.9,56.5,7.9c-0.4,0-0.8,0.2-1.2,0.6
			S54.8,9.5,54.8,10.3z"/>
		<path class="st3" d="M64.8,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S62.1,7,63,7c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S64.6,11.9,64.8,11.4z M61.3,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S61.4,9.2,61.3,9.7z"/>
		<path class="st3" d="M67.2,13.4V4.8h1.1v8.6C68.3,13.4,67.2,13.4,67.2,13.4z"/>
		<path class="st3" d="M69.9,6V4.8H71V6H69.9z M69.9,13.4V7.2H71v6.2H69.9z"/>
		<path class="st3" d="M72.5,13.4V7.2h0.9v0.9c0.5-0.7,1.1-1,2-1c0.4,0,0.7,0.1,1,0.2s0.5,0.3,0.7,0.5s0.3,0.5,0.3,0.8
			c0,0.2,0.1,0.5,0.1,1v3.8h-1.1V9.6c0-0.4,0-0.8-0.1-1S76.2,8.3,76,8.2S75.5,8,75.2,8c-0.4,0-0.8,0.1-1.2,0.4s-0.5,0.8-0.5,1.6v3.4
			H72.5z"/>
		<path class="st3" d="M83.5,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4S80.8,7,81.7,7c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3h-4.6c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S83.3,11.9,83.5,11.4z M80,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S80,9.2,80,9.7z"/>
		<path class="st3" d="M85.5,11.6l1-0.2c0.1,0.4,0.2,0.7,0.5,1s0.6,0.3,1.1,0.3s0.8-0.1,1.1-0.3s0.4-0.4,0.4-0.7
			c0-0.2-0.1-0.4-0.3-0.6c-0.1-0.1-0.5-0.2-1.1-0.4c-0.8-0.2-1.3-0.4-1.6-0.5S86,9.9,85.9,9.7s-0.2-0.5-0.2-0.8s0.1-0.5,0.2-0.8
			s0.3-0.4,0.5-0.6c0.2-0.1,0.4-0.2,0.7-0.3s0.6-0.1,0.9-0.1c0.5,0,0.9,0.1,1.3,0.2s0.6,0.3,0.8,0.6s0.3,0.6,0.4,1l-1,0.1
			c0-0.3-0.2-0.6-0.4-0.8s-0.5-0.3-1-0.3s-0.8,0.1-1,0.2c-0.2,0.1-0.3,0.3-0.3,0.6c0,0.1,0,0.3,0.1,0.4c0.1,0.1,0.2,0.2,0.4,0.3
			c0.1,0,0.4,0.1,0.9,0.3c0.7,0.2,1.3,0.4,1.6,0.5s0.5,0.3,0.7,0.6s0.3,0.5,0.3,0.9s-0.1,0.7-0.3,1s-0.5,0.6-0.9,0.7
			s-0.8,0.3-1.3,0.3c-0.8,0-1.4-0.2-1.8-0.5S85.6,12.2,85.5,11.6z"/>
	</g>
</g>
<g>
	<g>
		<path d="M106.3,14.5V9.1h-0.9V8.3h0.9V7.6c0-0.4,0-0.7,0.1-0.9c0.1-0.3,0.3-0.5,0.5-0.7s0.6-0.3,1.1-0.3c0.3,0,0.6,0,1,0.1
			l-0.2,0.9c-0.2,0-0.4-0.1-0.6-0.1c-0.3,0-0.5,0.1-0.7,0.2s-0.2,0.4-0.2,0.8v0.6h1.2V9h-1.2v5.4h-1
			C106.3,14.4,106.3,14.5,106.3,14.5z"/>
		<path d="M113.4,13.7c-0.4,0.3-0.8,0.6-1.1,0.7s-0.7,0.2-1.2,0.2c-0.7,0-1.2-0.2-1.6-0.5s-0.6-0.8-0.6-1.3c0-0.3,0.1-0.6,0.2-0.8
			s0.3-0.5,0.5-0.6s0.5-0.3,0.8-0.3c0.2-0.1,0.5-0.1,0.9-0.2c0.9-0.1,1.5-0.2,1.9-0.4c0-0.1,0-0.2,0-0.3c0-0.4-0.1-0.7-0.3-0.9
			c-0.3-0.2-0.7-0.4-1.2-0.4s-0.9,0.1-1.1,0.3s-0.4,0.5-0.5,0.9l-1-0.1c0.1-0.4,0.2-0.8,0.5-1.1s0.5-0.5,0.9-0.6s0.9-0.2,1.4-0.2
			s1,0.1,1.3,0.2s0.6,0.3,0.7,0.5s0.3,0.4,0.3,0.7c0,0.2,0.1,0.5,0.1,1v1.4c0,1,0,1.6,0.1,1.9s0.1,0.5,0.3,0.7h-1.1
			C113.5,14.3,113.4,14,113.4,13.7z M113.3,11.4c-0.4,0.2-1,0.3-1.7,0.4c-0.4,0.1-0.7,0.1-0.9,0.2s-0.3,0.2-0.4,0.3
			s-0.1,0.3-0.1,0.5c0,0.3,0.1,0.5,0.3,0.7s0.5,0.3,0.9,0.3s0.8-0.1,1.1-0.3s0.5-0.4,0.7-0.7c0.1-0.2,0.2-0.6,0.2-1.1L113.3,11.4
			L113.3,11.4z"/>
		<path d="M116,7.1V5.9h1.1v1.2C117.1,7.1,116,7.1,116,7.1z M116,14.5V8.3h1.1v6.2H116z"/>
		<path d="M118.6,14.5V5.9h1.1v8.6H118.6z"/>
		<path d="M125.6,12.5l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3H122c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S125.4,12.9,125.6,12.5z M122.1,10.8h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S122.2,10.2,122.1,10.8z"/>
		<path d="M132,14.5v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7c0-0.6,0.1-1.2,0.3-1.7
			c0.2-0.5,0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6v-3h1v8.6C132.9,14.5,132,14.5,132,14.5z
			 M128.7,11.4c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9S130.8,9,130.4,9
			c-0.5,0-0.9,0.2-1.2,0.6S128.7,10.6,128.7,11.4z"/>
	</g>
	<g>
		<path class="st3" d="M106.3,13.4V8h-0.9V7.2h0.9V6.5c0-0.4,0-0.7,0.1-0.9c0.1-0.3,0.3-0.5,0.5-0.7s0.6-0.3,1.1-0.3
			c0.3,0,0.6,0,1,0.1l-0.2,0.9c-0.2,0-0.4-0.1-0.6-0.1c-0.3,0-0.5,0.1-0.7,0.2s-0.2,0.4-0.2,0.8v0.6h1.2V8h-1.2v5.4H106.3z"/>
		<path class="st3" d="M113.4,12.6c-0.4,0.3-0.8,0.6-1.1,0.7s-0.7,0.2-1.2,0.2c-0.7,0-1.2-0.2-1.6-0.5s-0.6-0.8-0.6-1.3
			c0-0.3,0.1-0.6,0.2-0.8s0.3-0.5,0.5-0.6s0.5-0.3,0.8-0.3c0.2-0.1,0.5-0.1,0.9-0.2c0.9-0.1,1.5-0.2,1.9-0.4c0-0.1,0-0.2,0-0.3
			c0-0.4-0.1-0.7-0.3-0.9c-0.3-0.2-0.7-0.4-1.2-0.4s-0.9,0.1-1.1,0.3s-0.4,0.5-0.5,0.9l-1-0.1c0.1-0.4,0.2-0.8,0.5-1.1
			s0.5-0.5,0.9-0.6s0.9-0.2,1.4-0.2s1,0.1,1.3,0.2s0.6,0.3,0.7,0.5s0.3,0.4,0.3,0.7c0,0.2,0.1,0.5,0.1,1v1.4c0,1,0,1.6,0.1,1.9
			s0.1,0.5,0.3,0.7h-1.1C113.5,13.2,113.4,12.9,113.4,12.6z M113.3,10.3c-0.4,0.2-1,0.3-1.7,0.4c-0.4,0.1-0.7,0.1-0.9,0.2
			s-0.3,0.2-0.4,0.3s-0.1,0.3-0.1,0.5c0,0.3,0.1,0.5,0.3,0.7s0.5,0.3,0.9,0.3s0.8-0.1,1.1-0.3s0.5-0.4,0.7-0.7
			c0.1-0.2,0.2-0.6,0.2-1.1L113.3,10.3L113.3,10.3z"/>
		<path class="st3" d="M116,6V4.8h1.1V6H116z M116,13.4V7.2h1.1v6.2H116z"/>
		<path class="st3" d="M118.7,13.4V4.8h1.1v8.6C119.8,13.4,118.7,13.4,118.7,13.4z"/>
		<path class="st3" d="M125.6,11.4l1.1,0.1c-0.2,0.6-0.5,1.1-1,1.5s-1.1,0.5-1.8,0.5c-0.9,0-1.6-0.3-2.2-0.8s-0.8-1.3-0.8-2.4
			c0-1,0.3-1.9,0.8-2.4s1.2-0.9,2.1-0.9c0.8,0,1.5,0.3,2,0.8s0.8,1.4,0.8,2.4c0,0.1,0,0.2,0,0.3H122c0,0.7,0.2,1.2,0.6,1.6
			s0.8,0.5,1.3,0.5c0.4,0,0.7-0.1,1-0.3S125.4,11.9,125.6,11.4z M122.1,9.7h3.5c0-0.5-0.2-0.9-0.4-1.2c-0.3-0.4-0.8-0.6-1.3-0.6
			s-0.9,0.2-1.2,0.5S122.2,9.2,122.1,9.7z"/>
		<path class="st3" d="M132.1,13.4v-0.8c-0.4,0.6-1,0.9-1.7,0.9c-0.5,0-1-0.1-1.4-0.4s-0.7-0.7-1-1.1s-0.3-1.1-0.3-1.7
			c0-0.6,0.1-1.2,0.3-1.7c0.2-0.5,0.5-0.9,0.9-1.2s0.9-0.4,1.4-0.4c0.4,0,0.7,0.1,1,0.2s0.5,0.4,0.7,0.6v-3h1v8.6H132.1z
			 M128.7,10.3c0,0.8,0.2,1.4,0.5,1.8s0.7,0.6,1.2,0.6s0.9-0.2,1.2-0.6s0.5-1,0.5-1.7c0-0.9-0.2-1.5-0.5-1.9s-0.7-0.6-1.2-0.6
			s-0.9,0.2-1.2,0.6S128.7,9.5,128.7,10.3z"/>
	</g>
</g>
<g>
	<path d="M18,8c0-1.7-1.3-3-3-3h-0.5V4.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,4,7.5,4.1,7.5,4.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V6H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V9.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V10H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,17,7.6,17,7.7,17h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,10.9,18,9.6,18,8z"/>
	<path class="st3" d="M18,7c0-1.7-1.3-3-3-3h-0.5V3.2c0-0.1-0.1-0.2-0.2-0.2H7.7C7.6,3,7.5,3.1,7.5,3.2v2.7c0,0,0.1,0.1,0.2,0.1h6.7
		c0.1,0,0.2-0.1,0.2-0.2V5H15c1.1,0,2,0.9,2,2s-0.8,2-1.9,2c-0.1-0.2-0.4-0.3-0.6-0.3V8.2c0-0.1-0.1-0.2-0.2-0.2H7.6
		c0,0-0.1,0.1-0.1,0.2V9H7c-1.6,0-3,1.3-3,3c0,1.6,1.2,2.9,2.8,3c0.2,0.2,0.4,0.4,0.7,0.4l0,0v0.5C7.5,16,7.6,16,7.7,16h6.7
		c0.1,0,0.2-0.1,0.2-0.2v-2.7c0-0.1-0.1-0.2-0.2-0.2H7.7c-0.1,0-0.2,0.1-0.2,0.2v0.5l0,0c-0.3,0-0.5,0.1-0.7,0.3c-1-0.1-1.8-1-1.8-2
		c0-1.1,0.9-2,2-2h0.5v0.9c0,0.1,0.1,0.2,0.2,0.2h6.7c0.1,0,0.2-0.1,0.2-0.2v-0.5c0.3,0,0.5-0.2,0.7-0.4C16.7,9.9,18,8.6,18,7z"/>
</g>
</svg>
"""
    }
}