/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.tencent.devops.log.api

import com.tencent.devops.common.api.auth.AUTH_HEADER_DEVOPS_BUILD_ID
import com.tencent.devops.common.api.pojo.Result
import com.tencent.devops.common.log.pojo.message.LogMessage
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import io.swagger.annotations.ApiParam
import javax.ws.rs.Consumes
import javax.ws.rs.HeaderParam
import javax.ws.rs.POST
import javax.ws.rs.PUT
import javax.ws.rs.Path
import javax.ws.rs.Produces
import javax.ws.rs.QueryParam
import javax.ws.rs.core.MediaType

/**
 *
 * Powered By Tencent
 */
@Api(tags = ["BUILD_LOG"], description = "构建-日志资源")
@Path("/build/logs")
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
interface BuildLogResource {

    @ApiOperation("写入一条日志")
    @POST
    @Path("/")
    fun addLogLine(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("一条日志", required = true)
        logMessage: LogMessage
    ): Result<Boolean>

    @ApiOperation("写入一条红色高亮日志")
    @POST
    @Path("/red")
    fun addRedLogLine(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("一条日志", required = true)
        logMessage: LogMessage
    ): Result<Boolean>

    @ApiOperation("写入一条黄色高亮日志")
    @POST
    @Path("/yellow")
    fun addYellowLogLine(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("一条日志", required = true)
        logMessage: LogMessage
    ): Result<Boolean>

    @ApiOperation("写入多条日志")
    @POST
    @Path("/multi")
    fun addLogMultiLine(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("多条日志列表", required = true)
        logMessages: List<LogMessage>
    ): Result<Boolean>

    @ApiOperation("写入日志状态")
    @POST
    @Path("/status")
    fun addLogStatus(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("分辨插件的tag，默认填对应插件id", required = false)
        @QueryParam("tag")
        tag: String?,
        @ApiParam("插件内的分类的子tag，默认为空", required = false)
        @QueryParam("subTag")
        subTag: String?,
        @ApiParam("job id或者container的34位id", required = false)
        @QueryParam("jobId")
        jobId: String?,
        @ApiParam("执行次数", required = false)
        @QueryParam("executeCount")
        executeCount: Int?
    ): Result<Boolean>

    @ApiOperation("更新日志状态")
    @PUT
    @Path("/status")
    fun updateLogStatus(
        @ApiParam("构建ID", required = true)
        @HeaderParam(AUTH_HEADER_DEVOPS_BUILD_ID)
        buildId: String,
        @ApiParam("是否已构建完成", required = true)
        @QueryParam("finished")
        finished: Boolean,
        @ApiParam("分辨插件的tag，默认填对应插件id", required = false)
        @QueryParam("tag")
        tag: String?,
        @ApiParam("插件内的分类的子tag，默认为空", required = false)
        @QueryParam("subTag")
        subTag: String?,
        @ApiParam("job id或者container的34位id", required = false)
        @QueryParam("jobId")
        jobId: String?,
        @ApiParam("执行次数", required = false)
        @QueryParam("executeCount")
        executeCount: Int?
    ): Result<Boolean>
}