/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.tencent.devops.store.service.common.impl

import com.tencent.devops.common.api.pojo.Result
import com.tencent.devops.store.pojo.common.enums.StoreTypeEnum
import com.tencent.devops.store.service.common.StoreUserService
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Service

/**
 * store用户通用业务逻辑类
 *
 * since: 2019-03-26
 */
@Service
class SampleStoreUserServiceImpl : StoreUserService {

    private val logger = LoggerFactory.getLogger(SampleStoreUserServiceImpl::class.java)

    /**
     * 获取用户机构ID信息
     */
    override fun getUserDeptList(userId: String): List<Int> {
        return listOf() // 开源版用户暂无机构信息
    }

    /**
     * 获取用户机构名称
     */
    override fun getUserFullDeptName(userId: String): Result<String?> {
        return Result(data = "") // 开源版用户暂无机构信息
    }

    /**
     * 判断用户是否能安装store组件
     */
    override fun isCanInstallStoreComponent(
        defaultFlag: Boolean,
        userId: String,
        storeCode: String,
        storeType: StoreTypeEnum
    ): Boolean {
        logger.info("isCanInstallStoreComponent userId is :$userId,defaultFlag is :$defaultFlag")
        logger.info("isCanInstallStoreComponent storeCode is :$storeCode,storeType is :$storeType")
        return true // 开源版默认都有安装权限
    }
}
