/*
 * Tencent is pleased to support the open source community by making BK-CI 蓝鲸持续集成平台 available.
 *
 * Copyright (C) 2019 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-CI 蓝鲸持续集成平台 is licensed under the MIT license.
 *
 * A copy of the MIT License is included in this file.
 *
 *
 * Terms of the MIT License:
 * ---------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.tencent.devops.store.service.common

import com.tencent.devops.common.api.pojo.Result
import com.tencent.devops.common.pipeline.enums.ChannelCode
import com.tencent.devops.store.pojo.common.InstalledProjRespItem
import com.tencent.devops.store.pojo.common.enums.StoreTypeEnum

/**
 * store项目通用业务逻辑类
 *
 * since: 2019-03-22
 */
interface StoreProjectService {

    /**
     * 根据商店组件标识获取已安装的项目列表
     */
    fun getInstalledProjects(
        accessToken: String,
        userId: String,
        storeCode: String,
        storeType: StoreTypeEnum
    ): Result<List<InstalledProjRespItem>>

    /**
     * 安装商店组件
     */
    fun installStoreComponent(
        userId: String,
        projectCodeList: ArrayList<String>,
        storeId: String,
        storeCode: String,
        storeType: StoreTypeEnum,
        publicFlag: Boolean,
        channelCode: ChannelCode
    ): Result<Boolean>

    /**
     * 校验安装权限
     */
    fun validateInstallPermission(
        publicFlag: Boolean,
        userId: String,
        storeCode: String,
        storeType: StoreTypeEnum,
        projectCodeList: ArrayList<String>,
        channelCode: ChannelCode = ChannelCode.BS
    ): Result<Boolean>

    /**
     * 卸载商店组件
     */
    fun uninstall(
        storeType: StoreTypeEnum,
        storeCode: String,
        projectCode: String
    ): Result<Boolean>

    /**
     * 判断组件是否被项目安装
     */
    fun isInstalledByProject(
        projectCode: String,
        storeCode: String,
        storeType: Byte
    ): Boolean
}
