# 数据库脚本说明
## 脚本编写要求：

- 所有变更都要同步到ddl脚本中
- 要求：所有脚本必须是可反复执行无差别（幂等）：
  - 插入数据要求用INSERT IGNORE INTO 防止覆盖用户数据。
  - 对于需要强制刷新的系统数据可用更新语句 INSERT INTO xxx ON DUPLICATE KEY UPDATE 或者  REPLACE INTO  xxx
  - 不允许直接删除表或数据库后再重新创建的语句，只能判断后再创建：CREATE TABLE IF NOT EXISTS `表名`
  - 索引字段ALTER修改可用更新脚本存储过程，在update脚本中创建存储过程中在修改字段时先判断是否存在
- 不允许在程序不更新情况下无法运行：
  - 增加/修改字段，不允许将之设置为非空并且没有默认值
  - 不允许改名字段。
  - 删除表或字段操作，只需要更新ddl进行删除即可，不要提交update版本的脚本，防止用户误操作。


## 分类

- 创建数据库脚本：0001_{系统}_create-database_{db类型}.sql
  - 示例：0001_ci_create-database_mysql.sql 是所有db创建都放到该文件中
- 全版本的创建表/视图脚本(DDL)：1xxx_{系统}_{模块}_ddl_{db类型}.sql
  - 示例：1001_ci_process_ddl_mysql.sql  完全的db模型，如果有涉及到增字段，必须在该文件中补全，并在update中根据次版本
- 增量版本的DDL更新脚本(DDL update）2xxx~4xxx(预留)：2xxx_{系统}_{模块}_update_{次版本号}_{db类型}.sql
  - 示例：2001_ci_process_update_v2.1_mysql.sql  表示是 [2.1~2.2) 区间的所有版本的ddl更新
- 全版本的初始化数据脚本(DML)：5001_{系统}_{模块}_dml_{db类型}.sql 
  - 示例：5001_ci_process_dml_mysql.sql  表示是对process模块的初始化脚本，包含所有版本的内容。

**参数说明：**
**执行序号**： 4位数： 比如0001 按顺序下来，为方便指明执行顺序
**系统**： ci 表示是ci系统， codecc表示是codecc系统
**模块**:  process  project 等微服务模块
**版本号**： 1.1/1.2 等产品的版本号
**db类型**： mysql